from fastapi import FastAPI, Query, Request
from fastapi.responses import HTMLResponse
from fastapi.templating import Jinja2Templates
from starlette.staticfiles import StaticFiles
from rq_dashboard_fast.utils.jobs import QueueJobRegistryStats, get_jobs, JobDataDetailed, get_job, delete_job_id
from rq_dashboard_fast.utils.workers import WorkerData, get_workers
from rq_dashboard_fast.utils.queues import QueueRegistryStats, delete_jobs_for_queue, get_job_registry_amount
from pathlib import Path

class RedisQueueDashboard(FastAPI):
    def __init__(self, redis_url: str = "redis://localhost:6379", prefix: str = "/rq", *args, **kwargs):
        super().__init__(root_path=prefix, *args, **kwargs)
        
        package_directory = Path(__file__).resolve().parent
        static_directory = package_directory / "static"
        
        self.mount("/static", StaticFiles(directory=static_directory), name="static")

        templates_directory = package_directory / "templates"
        self.templates = Jinja2Templates(directory=templates_directory)
        self.redis_url = redis_url 

        self.rq_dashboard_version = "0.2.3" 

        @self.get("/", response_class=HTMLResponse)
        async def get_home(request: Request):
            return self.templates.TemplateResponse(
                "base.html",
                {"request": request, "active_tab": "jobs",
                 "instance_list": [], "prefix": prefix, "rq_dashboard_version": self.rq_dashboard_version}
            )

        @self.get("/workers", response_class=HTMLResponse)
        async def read_workers(request: Request):
            worker_data = get_workers(self.redis_url)

            active_tab = 'workers' 

            return self.templates.TemplateResponse(
                "workers.html",
                {"request": request, "worker_data": worker_data, "active_tab": active_tab,
                "instance_list": [], "prefix": prefix, "rq_dashboard_version": self.rq_dashboard_version}
            )
            
        @self.get("/workers/json", response_model=list[WorkerData])
        async def read_workers():
            worker_data = get_workers(self.redis_url)
            
            return worker_data
    
        @self.delete("/queues/{queue_name}")
        def delete_jobs_in_queue(queue_name: str):
            deleted_ids = delete_jobs_for_queue(queue_name, self.redis_url)
            return deleted_ids
            

        @self.get("/queues", response_class=HTMLResponse)
        async def read_queues(request: Request):
            queue_data = get_job_registry_amount(self.redis_url)

            active_tab = 'queues' 
            

            return self.templates.TemplateResponse(
                "queues.html",
                {"request": request, "queue_data": queue_data, "active_tab": active_tab,
                "instance_list": [], "prefix": prefix, "rq_dashboard_version": self.rq_dashboard_version}
            )
            
        @self.get("/queues/json", response_model=list[QueueRegistryStats])
        async def read_queues():
            queue_data = get_job_registry_amount(self.redis_url)

            return queue_data
    

        @self.get("/jobs", response_class=HTMLResponse)
        async def read_jobs(request: Request, queue_name: str = Query("all"), state: str = Query("all")):
            job_data = get_jobs(self.redis_url, queue_name, state)

            active_tab = 'jobs' 

            return self.templates.TemplateResponse(
                "jobs.html",
                {"request": request, "job_data": job_data, "active_tab": active_tab,
                "instance_list": [], "prefix": prefix, "rq_dashboard_version": self.rq_dashboard_version}
            )

        @self.get("/jobs/json", response_model=list[QueueJobRegistryStats])
        async def read_jobs(queue_name: str = Query("all"), state: str = Query("all")):
            job_data = get_jobs(self.redis_url, queue_name, state)

            return job_data

            
        @self.get("/job/{job_id}", response_model=JobDataDetailed)
        async def get_job_data(job_id: str, request: Request):
            job = get_job(self.redis_url, job_id)
            
            active_tab = "job"
            
            return self.templates.TemplateResponse(
                "job.html",
                {"request": request, "job_data": job, "active_tab": active_tab, 
                "instance_list": [], "prefix": prefix, "rq_dashboard_version": self.rq_dashboard_version}
            )
            
        @self.delete("/job/{job_id}")
        def delete_job(job_id: str):
            delete_job_id(self.redis_url, job_id=job_id)



import pytest
from fastapi.testclient import TestClient
from rq_dashboard_fast import RedisQueueDashboard
from rq import Queue
from redis import Redis
from random import randint
from fastapi import FastAPI
import time


# START REDIS ON PORT 8379 LOCALLY BEFORE RUNNING THIS

queue_name = "test_queue"

async def example_task() -> int:
    return randint(0, 100)

@pytest.fixture
def setup_redis():
    redis = Redis(port=8379)
    yield redis
    
@pytest.fixture
def client():
    app = FastAPI()
    dashboard = RedisQueueDashboard(redis_url="redis://localhost:8379", prefix="")

    app.mount("", dashboard)
    return TestClient(app)

@pytest.fixture
def create_queue(setup_redis):
    queue = Queue(connection=setup_redis, name="test_queue")
    return queue

@pytest.fixture
def add_task(create_queue):
    job = create_queue.enqueue(example_task)
    return job

def test_get_queue(client):
    response_read_html = client.get("/queues")
    assert response_read_html.status_code == 200
    assert queue_name in response_read_html.text

    response_read_json = client.get("/queues/json")
    assert response_read_json.status_code == 200
    assert any(queue["queue_name"] == queue_name for queue in response_read_json.json())

def test_add_job(client, add_task):
    job_id = add_task.id

    response = client.get("/job/" + job_id)
    assert job_id in response.text

    response_read_html = client.get("/jobs")
    assert response_read_html.status_code == 200
    assert job_id in response_read_html.text

    response_read_json = client.get("/jobs/json")
    assert response_read_json.status_code == 200
    assert job_id in response.text

def test_get_job_data(client, add_task):
    job_id = add_task.id

    response = client.get(f"/job/{job_id}")
    assert response.status_code == 200
    assert job_id in response.text
    
def test_get_jobs(client, add_task):
    job_id = add_task.id

    response = client.get("/jobs")
    assert response.status_code == 200
    assert job_id in response.text
    
    response = client.get("/jobs/json")
    assert response.status_code == 200
    assert job_id in response.text

def test_delete_job(client, add_task):
    job_id = add_task.id

    response_delete = client.delete(f"/job/{job_id}")
    assert response_delete.status_code == 200

    response_read_html_after_delete = client.get("/jobs")
    assert response_read_html_after_delete.status_code == 200
    assert job_id not in response_read_html_after_delete.text

def test_delete_jobs_in_queue(client, add_task):
    job_id = add_task.id

    response_delete = client.delete(f"/queues/{queue_name}")
    assert response_delete.status_code == 200

    response = client.get("/jobs/json")
    assert response.status_code == 200
    assert job_id not in response.text


from pydantic import BaseModel
from redis import Redis
from rq import Queue


class QueueRegistryStats(BaseModel):
    queue_name: str
    queued: int
    started: int
    failed: int
    deferred: int
    finished: int

def get_queues(redis_url: str) -> list[Queue]:
    redis = Redis.from_url(redis_url)
    
    queues = Queue.all(connection=redis)
    
    return queues
    

def get_job_registry_amount(redis_url: str) -> list[QueueRegistryStats]:
    queues = get_queues(redis_url)
    result = []
    for queue in queues:
        finished_jobs = len(queue.finished_job_registry.get_job_ids())      
        started_jobs = len(queue.started_job_registry.get_job_ids()) 
        failed_jobs = len(queue.failed_job_registry.get_job_ids()) 
        deferred_jobs = len(queue.deferred_job_registry.get_job_ids()) 
        queued_jobs = len(queue.get_job_ids()) 
                
        result.append(QueueRegistryStats(queue_name=queue.name, queued=queued_jobs, started=started_jobs, failed=failed_jobs, deferred=deferred_jobs, finished=finished_jobs))
    return result

def delete_jobs_for_queue(queue_name, redis_url) -> list[str]:
    redis = Redis.from_url(redis_url)
    
    queue = Queue(queue_name, connection=redis)
    
    result = queue.empty()
        
    return result


from datetime import datetime
from typing import Any, List
from fastapi import APIRouter, HTTPException
from pydantic import BaseModel
from redis import Redis
from rq.job import Job

from .queues import get_queues

router = APIRouter()
    
    
class JobData(BaseModel):
    id: str
    name: str
    created_at: datetime
    
class JobDataDetailed(BaseModel):
    id: str
    name: str
    created_at: datetime
    enqueued_at: datetime | None
    ended_at: datetime | None
    result: Any
    exc_info: str | None
    meta: dict

class QueueJobRegistryStats(BaseModel):
    queue_name: str
    queued: List[JobData]
    started: List[JobData]
    failed: List[JobData]
    deferred: List[JobData]
    finished: List[JobData]

def get_job_registrys(redis_url: str, queue_name: str = "all", state: str = "all"):
    redis = Redis.from_url(redis_url)
    
    queues = get_queues(redis_url)
    result = []
    
    for queue in queues:
        if queue_name == "all" or queue_name == queue.name:
            jobs = []

            if state == "all":
                jobs.extend(queue.get_job_ids())
                jobs.extend(queue.finished_job_registry.get_job_ids())
                jobs.extend(queue.failed_job_registry.get_job_ids())
                jobs.extend(queue.started_job_registry.get_job_ids())
                jobs.extend(queue.deferred_job_registry.get_job_ids())
            elif state == "queued":
                jobs.extend(queue.get_job_ids())
            elif state == "finished":
                jobs.extend(queue.finished_job_registry.get_job_ids())
            elif state == "failed":
                jobs.extend(queue.failed_job_registry.get_job_ids())
            elif state == "started":
                jobs.extend(queue.started_job_registry.get_job_ids())
            elif state == "deferred":
                jobs.extend(queue.deferred_job_registry.get_job_ids())

            jobs_fetched = Job.fetch_many(jobs, connection=redis)

            started_jobs = []
            failed_jobs = []
            deferred_jobs = []
            finished_jobs = []
            queued_jobs = []

            for job in jobs_fetched:
                status = job.get_status()
                if status == 'started':
                    started_jobs.append(JobData(id=job.id, name=job.description, created_at=job.created_at))
                elif status == 'failed':
                    failed_jobs.append(JobData(id=job.id, name=job.description, created_at=job.created_at))
                elif status == 'deferred':
                    deferred_jobs.append(JobData(id=job.id, name=job.description, created_at=job.created_at))
                elif status == 'finished':
                    finished_jobs.append(JobData(id=job.id, name=job.description, created_at=job.created_at))
                elif status == 'queued':
                    queued_jobs.append(JobData(id=job.id, name=job.description, created_at=job.created_at))

            result.append(QueueJobRegistryStats(queue_name=queue.name, queued=queued_jobs, started=started_jobs, failed=failed_jobs, deferred=deferred_jobs, finished=finished_jobs))
                
    return result


def get_jobs(redis_url: str, queue_name: str = None, state: str = None) -> list[QueueJobRegistryStats]:
    try:
        job_stats = get_job_registrys(redis_url, queue_name, state)
        return job_stats
    except Exception as e:
        # Handle specific exceptions if needed
        raise HTTPException(status_code=500, detail=str(e))

def get_job(redis_url: str, job_id: str) -> JobDataDetailed:
    redis = Redis.from_url(redis_url)
    job = Job.fetch(job_id, connection=redis)

    return JobDataDetailed(id=job.id, name=job.description, created_at=job.created_at, enqueued_at=job.enqueued_at, ended_at=job.ended_at, result=job.result, exc_info=job.exc_info, meta=job.meta)

def delete_job_id(redis_url: str, job_id: str):
    redis = Redis.from_url(redis_url)
    job = Job.fetch(job_id, connection=redis)
    if job:
        job.delete()



from pydantic import BaseModel
from redis import Redis
from rq import Worker

class WorkerData(BaseModel):
    name: str
    current_job: str | None
    queues: list[str]

def get_workers(redis_url: str) -> list[WorkerData]:
    redis = Redis.from_url(redis_url)
    workers = Worker.all(connection=redis)
    result = []
    
    for worker in workers:
        current_job = worker.get_current_job()
        if current_job is not None:
            result.append(WorkerData(name=worker.name, current_job=current_job.description, queues=worker.queue_names()))
        else:
            result.append(WorkerData(name=worker.name, current_job="Idle", queues=worker.queue_names()))
    
    return result